@@ -1,7 +1,7 @@
 /* tex-file.c: high-level file searching by format.
 
 Copyright (C) 1993, 94, 95, 96, 97 Karl Berry.
-Copyright 1998, 1998 Olaf Weber.
+Copyright 1998-2004 Olaf Weber.
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Library General Public
@@ -46,14 +46,14 @@ kpse_format_info_type kpse_format_info[kpse_last_format];
 #define PK_ENVS "PKFONTS", "TEXPKS", GLYPH_ENVS
 #define GLYPH_ENVS "GLYPHFONTS", "TEXFONTS"
 #define TFM_ENVS "TFMFONTS", "TEXFONTS"
-#define AFM_ENVS "AFMFONTS"
+#define AFM_ENVS "AFMFONTS", "TEXFONTS"
 #define BASE_ENVS "MFBASES", "TEXMFINI"
 #define BIB_ENVS "BIBINPUTS", "TEXBIB"
 #define BST_ENVS "BSTINPUTS"
 #define CNF_ENVS "TEXMFCNF"
 #define DB_ENVS "TEXMFDBS"
 #define FMT_ENVS "TEXFORMATS", "TEXMFINI"
-#define FONTMAP_ENVS "TEXFONTMAPS"
+#define FONTMAP_ENVS "TEXFONTMAPS", "TEXFONTS"
 #define MEM_ENVS "MPMEMS", "TEXMFINI"
 #define MF_ENVS "MFINPUTS"
 #define MFPOOL_ENVS "MFPOOL", "TEXMFINI"
@@ -74,16 +74,23 @@ kpse_format_info_type kpse_format_info[kpse_last_format];
 #define TEXSOURCE_ENVS "TEXSOURCES"
 #define TEX_PS_HEADER_ENVS "TEXPSHEADERS", "PSHEADERS"
 #define TROFF_FONT_ENVS "TRFONTS"
-#define TYPE1_ENVS "T1FONTS", "T1INPUTS", TEX_PS_HEADER_ENVS
+#define TYPE1_ENVS "T1FONTS", "T1INPUTS", "TEXFONTS", TEX_PS_HEADER_ENVS
 #define VF_ENVS "VFFONTS", "TEXFONTS"
 #define DVIPS_CONFIG_ENVS "TEXCONFIG"
 #define IST_ENVS "TEXINDEXSTYLE", "INDEXSTYLE"
-#define TRUETYPE_ENVS "TTFONTS"
-#define TYPE42_ENVS "T42FONTS"
+#define TRUETYPE_ENVS "TTFONTS", "TEXFONTS"
+#define TYPE42_ENVS "T42FONTS", "TEXFONTS"
 #define WEB2C_ENVS "WEB2C"
-#define MISCFONTS_ENVS "MISCFONTS"
+#define MISCFONTS_ENVS "MISCFONTS", "TEXFONTS"
 #define WEB_ENVS "WEBINPUTS"
 #define CWEB_ENVS "CWEBINPUTS"
+#define ENC_ENVS "ENCFONTS", "TEXFONTS"
+#define CMAP_ENVS "CMAPFONTS", "TEXFONTS"
+#define SFD_ENVS "SFDFONTS", "TEXFONTS"
+#define OPENTYPE_ENVS "OPENTYPEFONTS", "TEXFONTS"
+#define PDFTEXCONFIG_ENVS "PDFTEXCONFIG"
+#define LIG_ENVS "LIGFONTS", "TEXFONTS"
+#define TEXMFSCRIPTS_ENVS "TEXMFSCRIPTS"
 
 /* The compiled-in default list, DEFAULT_FONT_SIZES, is intended to be
    set from the command line (presumably via the Makefile).  */
@@ -95,7 +102,7 @@ kpse_format_info_type kpse_format_info[kpse_last_format];
 void
 kpse_init_fallback_resolutions P1C(string, envvar)
 {
-  string size, orig_size_list;
+  string size;
   const_string size_var = ENVVAR (envvar, "TEXSIZES");
   string size_str = getenv (size_var);
   unsigned *last_resort_sizes = NULL;
@@ -105,8 +112,6 @@ kpse_init_fallback_resolutions P1C(string, envvar)
                          : DEFAULT_FONT_SIZES; 
   string size_list = kpse_expand_default (size_str, default_sizes);
   
-  orig_size_list = size_list; /* For error messages.  */
-
   /* Initialize the list of last-resort sizes.  */
   for (size = kpse_path_element (size_list); size != NULL;
        size = kpse_path_element (NULL))
@@ -131,9 +136,7 @@ kpse_init_fallback_resolutions P1C(string, envvar)
   XRETALLOC (last_resort_sizes, size_count, unsigned);
   last_resort_sizes[size_count - 1] = 0;
 
-  /* If we didn't expand anything, we won't have allocated anything.  */
-  if (size_str && size_list != size_str)
-    free (size_list);
+  free (size_list);
     
   kpse_fallback_resolutions = last_resort_sizes;
 }
@@ -443,14 +446,15 @@ kpse_init_format P1C(kpse_file_format_type, format)
       break;
     case kpse_db_format:
       INIT_FORMAT ("ls-R", DEFAULT_TEXMFDBS, DB_ENVS);
-      SUFFIXES ("ls-R");
+#define LSR_SUFFIXES "ls-R", "ls-r"
+      SUFFIXES (LSR_SUFFIXES);
       FMT_INFO.path = remove_dbonly (FMT_INFO.path);
       break;
     case kpse_fmt_format:
       init_maketex (format, "mktexfmt", NULL);
       INIT_FORMAT ("fmt", DEFAULT_TEXFORMATS, FMT_ENVS);
       SUFFIXES (".fmt");
-#define FMT_SUFFIXES ".efmt", ".efm", ".oft"
+#define FMT_SUFFIXES ".efmt",".efm",".ofmt",".ofm",".oft",".eofmt",".eoft",".eof",".pfmt",".pfm",".epfmt",".epf",".xpfmt",".xpf",".afmt",".afm"
       ALT_SUFFIXES (FMT_SUFFIXES);
       FMT_INFO.binmode = true;
       break;
@@ -541,7 +545,7 @@ kpse_init_format P1C(kpse_file_format_type, format)
       INIT_FORMAT ("PostScript header", DEFAULT_TEXPSHEADERS,
                    TEX_PS_HEADER_ENVS);
 /* Unfortunately, dvipsk uses this format for type1 fonts.  */
-#define TEXPSHEADER_SUFFIXES ".pro", ".enc"
+#define TEXPSHEADER_SUFFIXES ".pro"
       ALT_SUFFIXES (TEXPSHEADER_SUFFIXES);
       FMT_INFO.binmode = true;
       break;
@@ -620,6 +624,32 @@ kpse_init_format P1C(kpse_file_format_type, format)
       SUFFIXES (CWEB_SUFFIXES);
       ALT_SUFFIXES (".ch");
       break;
+    case kpse_enc_format:
+      INIT_FORMAT ("enc files", DEFAULT_ENCFONTS, ENC_ENVS);
+      SUFFIXES (".enc");
+      break;
+    case kpse_cmap_format:
+      INIT_FORMAT ("cmap files", DEFAULT_CMAPFONTS, CMAP_ENVS);
+      SUFFIXES (".cmap");      
+      break;
+    case kpse_sfd_format:
+      INIT_FORMAT ("subfont definition files", DEFAULT_SFDFONTS, SFD_ENVS);
+      SUFFIXES (".sfd");
+      break;
+    case kpse_opentype_format:
+      INIT_FORMAT ("opentype fonts", DEFAULT_OPENTYPEFONTS, OPENTYPE_ENVS);
+      FMT_INFO.binmode = true;
+      break;
+    case kpse_pdftex_config_format:
+      INIT_FORMAT ("pdftex config", DEFAULT_PDFTEXCONFIG, PDFTEXCONFIG_ENVS);
+      break;
+    case kpse_lig_format:
+      INIT_FORMAT ("lig files", DEFAULT_LIGFONTS, LIG_ENVS);
+      SUFFIXES (".lig");
+      break;
+    case kpse_texmfscripts_format:
+      INIT_FORMAT ("texmfscripts", DEFAULT_TEXMFSCRIPTS, TEXMFSCRIPTS_ENVS);
+      break;
     default:
       FATAL1 ("kpse_init_format: Unknown format %d", format);
     }
@@ -693,6 +723,8 @@ kpse_find_file P3C(const_string, name,  kpse_file_format_type, format,
   boolean name_has_suffix_already = false;
   string mapped_name;
   string *mapped_names;
+  const_string *target;
+  unsigned count;
   boolean use_fontmaps = (format == kpse_tfm_format
                           || format == kpse_gf_format
                           || format == kpse_pk_format
@@ -706,6 +738,10 @@ kpse_find_file P3C(const_string, name,  kpse_file_format_type, format,
   if (FMT_INFO.path == NULL)
     kpse_init_format (format);
 
+  if (KPSE_DEBUG_P(KPSE_DEBUG_SEARCH))
+    DEBUGF3 ("kpse_find_file: searching for %s of type %s (from %s)\n",
+             name, FMT_INFO.type, FMT_INFO.path_source);
+
   /* Does NAME already end in a possible suffix?  */
   name_len = strlen (name);
   if (FMT_INFO.suffix) {
@@ -723,49 +759,93 @@ kpse_find_file P3C(const_string, name,  kpse_file_format_type, format,
     }
   }
 
-  /* Search #1: NAME doesn't have a suffix which is equal to a "standard"
+  /* Set up target list. */
+  count = 0;
+  target = XTALLOC1 (const_string);
+  /* Case #1: NAME doesn't have a suffix which is equal to a "standard"
      suffix.  For example, foo.bar, but not foo.tex.  We look for the
      name with the standard suffixes appended. */
   if (!name_has_suffix_already && FMT_INFO.suffix) {
-    /* Append a suffix and search for it.   Don't pound the disk yet.  */
-    for (ext = FMT_INFO.suffix; !ret && *ext; ext++) {
+    for (ext = FMT_INFO.suffix; *ext; ext++) {
       string name_with_suffix = concat (name, *ext);
-      ret = kpse_path_search (FMT_INFO.path, name_with_suffix, false);
-      if (!ret && use_fontmaps) {
-        mapped_names = kpse_fontmap_lookup (name_with_suffix);
-        while (mapped_names && (mapped_name = *mapped_names++) && !ret)
-          ret = kpse_path_search (FMT_INFO.path, mapped_name, false);
-      }
-      free (name_with_suffix);
-    }
-    /* If we only do suffix searches, and are instructed to pound the disk,
-       do so now.  We don't pound the disk for aliases...perhaps we should? */
-    if (!ret && FMT_INFO.suffix_search_only && must_exist) {
-      for (ext = FMT_INFO.suffix; !ret && *ext; ext++) {
-        string name_with_suffix = concat (name, *ext);
-        ret = kpse_path_search (FMT_INFO.path, name_with_suffix, true);
-        free (name_with_suffix);
+      target[count++] = name_with_suffix;
+      XRETALLOC(target, count+1, const_string);
+      if (use_fontmaps
+          && (mapped_names = kpse_fontmap_lookup (name_with_suffix)) != NULL)
+      {
+        /* FIXME: we leak mapped_names and its elements, some of the time */
+        while ((mapped_name = *mapped_names++) != NULL) {
+          target[count++] = xstrdup (mapped_name);
+          XRETALLOC(target, count+1, const_string);
+        }
       }
     }
   }
-
-  /* Search #2: Just look for the name we've been given, provided non-suffix
+  /* Case #2: Just look for the name we've been given, provided non-suffix
      searches are allowed or the name already includes a suffix. */
-  if (!ret && (name_has_suffix_already || !FMT_INFO.suffix_search_only)) {
-    ret = kpse_path_search (FMT_INFO.path, name, false);
-    if (!ret && use_fontmaps) {
-      mapped_names = kpse_fontmap_lookup (name);
-      while (mapped_names && (mapped_name = *mapped_names++) && !ret)
-        ret = kpse_path_search (FMT_INFO.path, mapped_name, false);
-    }
-    /* We don't pound the disk for aliases...perhaps we should? */
-    if (!ret && must_exist) {
-      ret = kpse_path_search (FMT_INFO.path, name, true);
+  if (name_has_suffix_already || !FMT_INFO.suffix_search_only) {
+    target[count++] = xstrdup (name);
+    XRETALLOC(target, count+1, const_string);
+    if (use_fontmaps && (mapped_names = kpse_fontmap_lookup (name)) != NULL) {
+      /* FIXME: we leak mapped_names and its elements, some of the time */
+      while ((mapped_name = *mapped_names++) != NULL) {
+        target[count++] = xstrdup (mapped_name);
+        XRETALLOC(target, count+1, const_string);
+      }
     }
   }
+  /* Terminate list. */
+  target[count] = NULL;
 
-  /* Search #3 (sort of): Call mktextfm or whatever to create a
-     missing file.  */
+  /* Search. */
+#if 0
+  /* Simple version, just do the search for the list, and pound disk
+   * then and there if appropriate.
+   */
+  ret = kpse_path_search_list (FMT_INFO.path, target, must_exist);
+#elif 0
+  /* Variant where we delay disk-pounding. */
+  ret = kpse_path_search_list (FMT_INFO.path, target, false);
+  if (!ret && must_exist)
+      ret = kpse_path_search_list (FMT_INFO.path, target, true);
+#else
+  /* Variant that tries the match the original behaviour more closely.
+   * The distinction is that we pound the disk for a far shorter list
+   * of names.
+   *
+   * This is the fastest variant, because it does fewer disk accesses
+   * than the two variants above -- for example some 750 calls of
+   * access(2) compared to 1500.
+   */
+  ret = kpse_path_search_list (FMT_INFO.path, target, false);
+  /* Do we need to pound the disk? */
+  if (!ret && must_exist) {
+    for (count = 0; target[count]; count++)
+      free((void*)target[count]);
+    count = 0;
+    /* We look for a subset of the previous set of names, so the
+     * target array is large enough.  In particular, we don't pound
+     * the disk for alternate names from the fontmaps.  Perhaps we
+     * should?
+     */
+    if (!name_has_suffix_already && FMT_INFO.suffix_search_only) {
+      for (ext = FMT_INFO.suffix; !ret && *ext; ext++)
+        target[count++] = concat (name, *ext);
+    }
+    if (name_has_suffix_already || !FMT_INFO.suffix_search_only) {
+      target[count++] = xstrdup (name);
+    }
+    target[count] = NULL;
+    ret = kpse_path_search_list (FMT_INFO.path, target, true);
+  }
+#endif  
+  
+  /* Free the list we created. */
+  for (count = 0; target[count]; count++)
+    free((void*)target[count]);
+  free(target);
+  
+  /* If nothing was found, call mktex* to create a missing file.  */
   if (!ret && must_exist) {
     ret = kpse_make_tex (format, name);
   }
@@ -818,6 +898,7 @@ kpse_reset_program_name P1C(const_string, progname)
 
   free (kpse_program_name);
   kpse_program_name = xstrdup (progname);
+  xputenv("progname", kpse_program_name);
   
   /* Clear paths -- do we want the db path to be cleared? */
   for (i = 0; i != kpse_last_format; ++i) {
